%                                                                  %
% CREATE - CASE STUDY - SUMMER 2019 - MCMASTER UNIVERSITY          %
% Author: Christina Riczu                                          %
% Date: 20/08/2019                                                 %
%                                                                  %
% This case study automates the validation of fittings after the   %
% SE14 fitting deflection test. It takes generic, standard Instron %
% files as input, formats the position and force into correct sign %
% and standard SI units, and outputs to a data file. It will also  %
% generate charts for the BMW reports.                             %

close all 
clear all
clc

%% ENTER FILE DIRECTORY FOR INSTRON FILES

% replace the line of code below with the folder location of this .m file
fileDir = ['C:\Users\chris\OneDrive\Desktop\Masters - Passport\Research\CMHT Data\Case Study\matlab program\Instron Files','\'];
folderInfo = dir(fileDir);

%% INITIALIZATIONS
num_files = length(folderInfo) - 2;
filenames = strings(num_files,1);
Nfilenames = strings(num_files,1);
Pfilenames = strings(num_files,1);
datapoints = zeros(num_files,1);
rawData = zeros(65535, num_files * 3);

%% GATHER INPUTS INTO ARRAY

for k=3:length(folderInfo)
    % get filename
    filename = folderInfo(k).name; % this includes the .xls at the end
    filenames(k-2) = filename;
    % read in data, cut to number of datapoints
    T = xlsread(join([fileDir,filename]),'Curve_Table','C2:E65536');
    datapoint = length(T); %find(T(:,1)==0, 1, 'first');
    datapoints(k-2) = datapoint;
    % generate raw dataset
    rawData(1:datapoint,(3*k-8):(3*k-6))=T;
end
truData = rawData;

%% RE-FORMAT THE FORCE DATA

for i=1:num_files
    % flip sign of force data because Instron records it flipped
    truData(1:datapoints(i),3*i) = -1 * truData(1:datapoints(i),3*i);
    ave = mean(truData(1:datapoints(i),3*i));
    % normalize / center
    truData(1:datapoints(i),3*i) = truData(1:datapoints(i),3*i) - ave;
end

%% CONVERT TO SI UNITS

for i=1:num_files
    % convert inches to mm
    truData(1:datapoints(i),3*i-1) = truData(1:datapoints(i),3*i-1) * 25.4;
    % convert lbs-ft to Nm
    truData(1:datapoints(i),3*i) = truData(1:datapoints(i),3*i) * 4.44822 * 0.036;
end

%% FIND RANGES FOR REPORT

% find the maximum value in each column
maximums = max(truData); 
% find the minimum value in each column
minimums = min(truData); 

max_time = maximums(1:3:end)';
max_pos = maximums(2:3:end)';
max_for = maximums(3:3:end)';

min_time = minimums(1:3:end)';
min_pos = minimums(2:3:end)';
min_for = minimums(3:3:end)';

%% SORT DATA INTO NORMAL AND PARALLEL TESTS

% separate the file name into text cols
c = strings(num_files,5);
for n=1:length(filenames)
    c(n,:) = strsplit(filenames(n)," ");
    temp = strsplit(c(n,5),".");
    c(n,5) = temp{1};
end

%% GENERATE PLOTS FOR REPORT

% find n and p indexes
n_ind = find(c(:,5)=="N");
p_ind = find(c(:,5)=="P");

% n pos
x = 1:26; %c(n_ind,1);
y1 = max_pos(n_ind); 
y2 = min_pos(n_ind);
figure(1)
hold on
scatter(x,y1,'b','filled');
scatter(x,y2,'r','filled');
title("Normal Test - Position");
xlabel("Test Number");ylabel("Deflection (mm)");
xlim([0 26]);ylim([-1.5 1.5]);
grid on

% n for
x = 1:26; %c(n_ind,1);
y1 = max_for(n_ind); 
y2 = min_for(n_ind);
figure(2)
hold on
scatter(x,y1,'b','filled');
scatter(x,y2,'r','filled');
title("Normal Test - Force");
xlabel("Test Number");ylabel("Force (Nm)");
xlim([0 26]);ylim([-2 2]);
grid on

% P pos
x = 1:26; %c(n_ind,1);
y1 = max_pos(p_ind); 
y2 = min_pos(p_ind);
figure(3)
hold on
scatter(x,y1,'b','filled');
scatter(x,y2,'r','filled');
title("Parallel Test - Position");
xlabel("Test Number");ylabel("Deflection (mm)");
xlim([0 26]);ylim([-1.5 1.5]);
grid on

% P for
x = 1:26; %c(n_ind,1);
y1 = max_for(p_ind); 
y2 = min_for(p_ind);
figure(4)
hold on
scatter(x,y1,'b','filled');
scatter(x,y2,'r','filled');
title("Parallel Test - Force");
xlabel("Test Number");ylabel("Force (Nm)");
xlim([0 26]);ylim([-10 10]);
grid on


%% OUTPUT DATA TO CSV AND MAT

save dataFile.mat rawData truData

%% SAVE FIGURES

h = get(0,'children');
len = length(h);
for i=1:length(h)
  saveas(h(i), ['figure' num2str(len-i+1) ], 'jpg');
end
